@@ -12,6 +12,7 @@
 //! ```
 
 use bitcoincore_rpc::{Auth, Client, RpcApi};
+use log::debug;
 use std::ffi::OsStr;
 use std::net::{Ipv4Addr, SocketAddrV4, TcpListener};
 use std::path::PathBuf;
@@ -32,6 +33,16 @@ pub struct BitcoinD {
     /// Work directory, where the node store blocks and other stuff. It is kept in the struct so that
     /// directory is deleted only when this struct is dropped
     _work_dir: TempDir,
+
+    /// Node configuration, contains information to connect to this node
+    pub config: Config,
+}
+
+#[derive(Debug, Clone)]
+/// Contains all the information to connect to this node
+pub struct Config {
+    /// Path to the node datadir
+    pub datadir: PathBuf,
     /// Path to the node cookie file, useful for other client to connect to the node
     pub cookie_file: PathBuf,
     /// Url of the rpc of the node, useful for other client to connect to the node
@@ -75,14 +86,19 @@ impl BitcoinD {
     /// `p2p` allows to specify options to open p2p port or connect to the another node
     /// `datadir` when None a temp directory is created as datadir, it will be deleted on drop
     ///  provide a directory when you don't want auto deletion (maybe because you can't control
-    pub fn with_args<S, I>(exe: S, args: I, view_stdout: bool, p2p: P2P) -> Result<BitcoinD, Error>
+    pub fn with_args<S, I>(
+        exe: S,
+        custom_args: I,
+        view_stdout: bool,
+        p2p: P2P,
+    ) -> Result<BitcoinD, Error>
     where
         I: IntoIterator<Item = S>,
         S: AsRef<OsStr>,
     {
         let _work_dir = TempDir::new()?;
-        let datadir_path = _work_dir.path().to_path_buf();
-        let cookie_file = datadir_path.join("regtest").join(".cookie");
+        let datadir = _work_dir.path().to_path_buf();
+        let cookie_file = datadir.join("regtest").join(".cookie");
         let rpc_port = get_available_port()?;
         let rpc_socket = SocketAddrV4::new(LOCAL_IP, rpc_port);
         let rpc_url = format!("http://{}", rpc_socket);
@@ -110,13 +126,20 @@ impl BitcoinD {
             Stdio::null()
         };
 
+        let datadir_arg = format!("-datadir={}", datadir.display());
+        let rpc_arg = format!("-rpcport={}", rpc_port);
+        let default_args = [&datadir_arg, &rpc_arg, "-regtest", "-fallbackfee=0.0001"];
+
+        debug!(
+            "launching {:?} with args: {:?} {:?} AND custom args",
+            exe.as_ref(),
+            default_args,
+            p2p_args
+        );
         let process = Command::new(exe)
-            .arg(format!("-datadir={}", datadir_path.display()))
-            .arg(format!("-rpcport={}", rpc_port))
-            .arg("-regtest")
-            .arg("-fallbackfee=0.0001")
-            .args(p2p_args)
-            .args(args)
+            .args(&default_args)
+            .args(&p2p_args)
+            .args(custom_args)
             .stdout(stdout)
             .spawn()?;
 
@@ -141,15 +164,23 @@ impl BitcoinD {
             process,
             client,
             _work_dir,
-            cookie_file,
-            rpc_socket,
-            p2p_socket,
+            config: Config {
+                datadir,
+                cookie_file,
+                rpc_socket,
+                p2p_socket,
+            },
         })
     }
 
     /// Returns the rpc URL including the schema eg. http://127.0.0.1:44842
-    pub fn rpc_url(&self) -> Result<String, Error> {
-        Ok(format!("http://{}", self.rpc_socket))
+    pub fn rpc_url(&self) -> String {
+        format!("http://{}", self.config.rpc_socket)
+    }
+
+    /// Returns the [P2P] enum to connect to this node p2p port
+    pub fn p2p_connect(&self) -> Option<P2P> {
+        self.config.p2p_socket.map(P2P::Connect)
     }
 
     /// Stop the node, waiting correct process termination
@@ -204,7 +235,7 @@ mod test {
 
     #[test]
     fn test_bitcoind() {
-        let exe = env::var("BITCOIND_EXE").expect("BITCOIND_EXE env var must be set");
+        let exe = init();
         let bitcoind = BitcoinD::new(exe).unwrap();
         let info = bitcoind.client.get_blockchain_info().unwrap();
         assert_eq!(0, info.blocks);
@@ -216,7 +247,7 @@ mod test {
 
     #[test]
     fn test_getindexinfo() {
-        let exe = env::var("BITCOIND_EXE").expect("BITCOIND_EXE env var must be set");
+        let exe = init();
         let bitcoind =
             BitcoinD::with_args(exe, vec!["-txindex".to_string()], false, P2P::No).unwrap();
         assert!(
@@ -230,17 +261,17 @@ mod test {
 
     #[test]
     fn test_p2p() {
-        let exe = env::var("BITCOIND_EXE").expect("BITCOIND_EXE env var must be set");
+        let exe = init();
         let bitcoind = BitcoinD::with_args(exe.clone(), vec![], false, P2P::Yes).unwrap();
         assert_eq!(bitcoind.client.get_peer_info().unwrap().len(), 0);
-        let other_bitcoind = BitcoinD::with_args(
-            exe,
-            vec![],
-            false,
-            P2P::Connect(bitcoind.p2p_socket.clone().unwrap()),
-        )
-        .unwrap();
+        let other_bitcoind =
+            BitcoinD::with_args(exe, vec![], false, bitcoind.p2p_connect().unwrap()).unwrap();
         assert_eq!(bitcoind.client.get_peer_info().unwrap().len(), 1);
         assert_eq!(other_bitcoind.client.get_peer_info().unwrap().len(), 1);
     }
+
+    fn init() -> String {
+        let _ = env_logger::try_init();
+        env::var("BITCOIND_EXE").expect("BITCOIND_EXE env var must be set")
+    }
 }
